/* mlm_remora.c
 *
 * CallCommander Integration for a Logitek Remora control surface.
 *
 *   (C) Copyright 2002-2007 Fred Gleason <fredg@paravelsystems.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This module displays call screening data on the LCD displays of
 * a Logitek Remora control surface.
 *
 * To compile this module, just do:
 * 
 *   gcc -shared -o mlm_remora.mlm mlm_remora.c
 */

/*
 * Site Definitions -- Customize these values for your site!
 */
#define SUPERVISOR_ADDRESS "supervisor0.radioamerica.org"
#define VGUEST_PROFILE_NAME "cc-b"
#define VGUEST_PROFILE_PASSWORD "letmein"
#define VGUEST_HYBRID0_ENGINE 3
#define VGUEST_HYBRID0_DEVICE 0x0105
#define VGUEST_HYBRID1_ENGINE 3
#define VGUEST_HYBRID1_DEVICE 0x0106
#define VGUEST_FADER1_ENGINE 3
#define VGUEST_FADER1_DEVICE 0x000B
#define VGUEST_CLOCK_ENGINE 3
#define VGUEST_CLOCK_DEVICE 0x0017
#define VGUEST_CLOCK_LINE 48
#define VGUEST_CLOCK_COLUMN 0
#define DELAY_LOW_LIMIT 3000

/*
 * **************************************************************************
 *  You should not need to change anything below here!
 * **************************************************************************
 */

#define FLASH_TIME_ON 600
#define FLASH_TIME_OFF 400

#include <stdio.h>
#include <string.h>
#include <mlm/mlm.h>

/*
 * Globals
 */
int vguest_handle;
char caller_name[12][256];
char caller_city[12][256];
char caller_state[12][256];
char caller_station[12][256];
char caller_comment[12][1024];
char caller_textline[12][2][2048];
int caller_lstate[12];
int line_bank[12];
int delay_offset;
char predelay_clock_string[20];
char postdelay_clock_string[20];
int delay_warning_on;

void WriteLineEntry(int line)
{
  char label[1024];

  /*
   * First Text Line -- Name, City, State, Station
   */
  if(strlen(caller_name[line])>0) {
    sprintf(label,"%s ",caller_name[line]);
  }
  else {
    switch(caller_lstate[line]) {
	case MLM_LINE_IDLE:
	case MLM_LINE_INACTIVE:
	  label[0]=0;
	  break;

	default:
	  sprintf(label,"Name Unknown ");
	  break;
    }
  }
  if((strlen(caller_city[line])>0)||(strlen(caller_state[line])>0)) {
    sprintf(label+strlen(label),"- %s %s ",caller_city[line],caller_state[line]);
  }
  if(strlen(caller_station[line])>0) {
    sprintf(label+strlen(label),"- %s",caller_station[line]);
  }
  MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
		     VGUEST_FADER1_DEVICE+line,0,0,0,0,label);

  /*
   * Second Text Line -- Comment
   */
  MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
		     VGUEST_FADER1_DEVICE+line,0,1,0,0,caller_comment[line]);

  /*
   * Update Text Lines
   */
  strncpy(caller_textline[line][0],label,2048);
  strncpy(caller_textline[line][1],caller_comment[line],2048);
}


void ClearLineEntry(int line)
{
  int j;

  caller_name[line][0]=0;
  caller_city[line][0]=0;
  caller_state[line][0]=0;
  caller_station[line][0]=0;
  caller_comment[line][0]=0;
  for(j=0;j<2;j++) {
    caller_textline[line][j][0]=0;
  }
}


void MLMStartup()
{
  int i;
  int j;

  /*
   * Initialize Globals
   */
  delay_offset=0;
  predelay_clock_string[0]=0;
  postdelay_clock_string[0]=0;
  delay_warning_on=0;
  for(i=0;i<12;i++) {
    caller_name[i][0]=0;
    caller_city[i][0]=0;
    caller_state[i][0]=0;
    caller_station[i][0]=0;
    caller_comment[i][0]=0;
    caller_lstate[i]=MLM_LINE_INACTIVE;
    line_bank[i]=-1;
    for(j=0;j<2;j++) {
      caller_textline[i][j][0]=0;
    }
  }

  /*
   * Start Timers
   */
  MLMStartTimer(0,FLASH_TIME_ON,MLM_TIMER_ONESHOT); /* On-Air ON Flash Timer */
  MLMStartTimer(2,100,MLM_TIMER_REPEATING);         /* Clock Poll Timer */
  MLMStartTimer(3,1000,MLM_TIMER_REPEATING);  /* Delay Warning Flash Timer */

  /*
   * Initialize vGuest Connection
   */
  vguest_handle=MLMVGuestOpen(SUPERVISOR_ADDRESS,MLM_VGUEST_DEFAULT_TCP_PORT,
			      VGUEST_PROFILE_NAME,VGUEST_PROFILE_PASSWORD);
}


void MLMShutdown()
{
  int i;

  /*
   * Clear Line Info Area
   */
  for(i=0;i<4;i++) {
    MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
		       VGUEST_FADER1_DEVICE,0,i+48,0,
		       MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
		       MLM_VGUEST_TEXT_VIDEO_NORMAL|
		       MLM_VGUEST_TEXT_COLOR_WHITE," ");
    MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
		       VGUEST_FADER1_DEVICE+6,0,i+48,0,
		       MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
		       MLM_VGUEST_TEXT_VIDEO_NORMAL|
		       MLM_VGUEST_TEXT_COLOR_WHITE," ");
  }

  /*
   * Clear Clocks
   */
  for(i=0;i<4;i++) {
    MLMVGuestWriteText(vguest_handle,VGUEST_CLOCK_ENGINE,
		       VGUEST_CLOCK_DEVICE,0,
		       VGUEST_CLOCK_LINE+i,
		       VGUEST_CLOCK_COLUMN,
		       MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
		       MLM_VGUEST_TEXT_VIDEO_NORMAL|
		       MLM_VGUEST_TEXT_COLOR_WHITE," ");
  }
}


void MLMLineStateChanged(int lstate,int line,int bank,int console)
{
  char label[8];

  sprintf(label,"Line %2d",line+1);
  switch(lstate) {
      case MLM_LINE_ONAIR:
      case MLM_LINE_LOCKED:
	/*
	 * Set Fader Label
	 */
	switch(bank) {
	    case 0:
	      MLMVGuestWriteText(vguest_handle,VGUEST_HYBRID0_ENGINE,
				 VGUEST_HYBRID0_DEVICE,0,2,0,0,label);
	      break;

	    case 1:
	      MLMVGuestWriteText(vguest_handle,VGUEST_HYBRID1_ENGINE,
				 VGUEST_HYBRID1_DEVICE,0,2,0,0,label);
	      break;
	}
	line_bank[line]=bank;
	break;

      default:
	/*
	 * Clear Fader Label
	 */
	switch(line_bank[line]) {
	    case 0:
	      MLMVGuestWriteText(vguest_handle,VGUEST_HYBRID0_ENGINE,
				 VGUEST_HYBRID0_DEVICE,0,2,0,0," ");
	      break;

	    case 1:
	      MLMVGuestWriteText(vguest_handle,VGUEST_HYBRID1_ENGINE,
				 VGUEST_HYBRID1_DEVICE,0,2,0,0," ");
	      break;
	}
	line_bank[line]=-1;
	break;
  }

  switch(lstate) {
      case MLM_LINE_INACTIVE:
      case MLM_LINE_IDLE:
	/*
	 * Clear Line Label
	 */
	ClearLineEntry(line);
	MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
			   VGUEST_FADER1_DEVICE+line,0,0,0,0," ");
	MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
			   VGUEST_FADER1_DEVICE+line,0,1,0,0," ");
	break;

      default:
	WriteLineEntry(line);
	break;
  }
  caller_lstate[line]=lstate;
}


void MLMDelayOffsetChanged(int offset)
{
  printf("MLMDelayOffsetReceived(%d)\n",offset);
  delay_offset=offset;
}


void MLMCallerElementReceived(int line,int elem,const char *str)
{
  switch(elem) {
      case MLM_ELEMENT_NAME:
	strncpy(caller_name[line],str,256);
	break;

      case MLM_ELEMENT_CITY:
	strncpy(caller_city[line],str,256);
	break;

      case MLM_ELEMENT_STATE:
	strncpy(caller_state[line],str,256);
	break;

      case MLM_ELEMENT_STATION:
	strncpy(caller_station[line],str,256);
	break;

      case MLM_ELEMENT_COMMENT:
	strncpy(caller_comment[line],str,1024);
	break;

      default:
	return;
  }
  WriteLineEntry(line);
}


void MLMTimerExpired(int timernum)
{
  int i;
  int j;
  char buffer[256];

  switch(timernum) {
    case 0:  /* On-Air ON Timer */
      for(i=0;i<12;i++) {
	switch(caller_lstate[i]) {
	case MLM_LINE_ONAIR:
	case MLM_LINE_LOCKED:
	  for(j=0;j<2;j++) {
	    MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
			       VGUEST_FADER1_DEVICE+i,0,j,0,0,
			       caller_textline[i][j]);
	  }
	  break;
	}
      }
      MLMStartTimer(1,FLASH_TIME_ON,MLM_TIMER_ONESHOT);  /* OFF Timer */
      break;

    case 1:  /* On-Air OFF Timer */
      for(i=0;i<12;i++) {
	switch(caller_lstate[i]) {
	case MLM_LINE_ONAIR:
	case MLM_LINE_LOCKED:
	  for(j=0;j<2;j++) {
	    MLMVGuestWriteText(vguest_handle,VGUEST_FADER1_ENGINE,
			       VGUEST_FADER1_DEVICE+i,0,j,0,0," ");
	  }
	  break;
	}
      }
      MLMStartTimer(0,FLASH_TIME_OFF,MLM_TIMER_ONESHOT);  /* ON Timer */
      break;

    case 2:  /* Poll Clock */
      if((strcmp(MLMDateTime(-delay_offset,"hh:mm:ss"),
		 predelay_clock_string))||
	 (strcmp(MLMDateTime(0,"hh:mm:ss"),
		 postdelay_clock_string))!=0) {
	strcpy(predelay_clock_string,MLMDateTime(delay_offset,"hh:mm:ss"));
	strcpy(postdelay_clock_string,MLMDateTime(0,"hh:mm:ss"));
	sprintf(buffer,"      %s     %s",predelay_clock_string,
		postdelay_clock_string);
	MLMVGuestWriteText(vguest_handle,VGUEST_CLOCK_ENGINE,
			   VGUEST_CLOCK_DEVICE,0,
			   VGUEST_CLOCK_LINE,
			   VGUEST_CLOCK_COLUMN,
			   MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
			   MLM_VGUEST_TEXT_VIDEO_NORMAL|
			   MLM_VGUEST_TEXT_COLOR_WHITE,
			   "      ---- CURRENT TIME ----");
	MLMVGuestWriteText(vguest_handle,VGUEST_CLOCK_ENGINE,
			   VGUEST_CLOCK_DEVICE,0,
			   VGUEST_CLOCK_LINE+1,
			   VGUEST_CLOCK_COLUMN,
			   MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
			   MLM_VGUEST_TEXT_VIDEO_NORMAL|
			   MLM_VGUEST_TEXT_COLOR_WHITE,
			   "      Pre-Delay   Post-Delay");
	MLMVGuestWriteText(vguest_handle,VGUEST_CLOCK_ENGINE,
			   VGUEST_CLOCK_DEVICE,0,
			   VGUEST_CLOCK_LINE+2,
			   VGUEST_CLOCK_COLUMN,
			   MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
			   MLM_VGUEST_TEXT_VIDEO_NORMAL|
			   MLM_VGUEST_TEXT_COLOR_WHITE,
			   buffer);
      }
      break;

    case 3:  /* Delay Warning */
      switch(delay_warning_on) {
        case 0:
	  if(delay_offset<DELAY_LOW_LIMIT) {
	    sprintf(buffer," WARNING: Delay below :%02d seconds!",
		    DELAY_LOW_LIMIT/1000);
	    MLMVGuestWriteText(vguest_handle,VGUEST_CLOCK_ENGINE,
			       VGUEST_CLOCK_DEVICE,0,
			       VGUEST_CLOCK_LINE+3,
			       VGUEST_CLOCK_COLUMN,
			       MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
			       MLM_VGUEST_TEXT_VIDEO_NORMAL|
			       MLM_VGUEST_TEXT_COLOR_WHITE,
			       buffer);
	    delay_warning_on=1;
	  }
	  break;

        case 1:
	  MLMVGuestWriteText(vguest_handle,VGUEST_CLOCK_ENGINE,
			     VGUEST_CLOCK_DEVICE,0,
			     VGUEST_CLOCK_LINE+3,
			     VGUEST_CLOCK_COLUMN,
			     MLM_VGUEST_TEXT_DISPLAY_OVERWRITE|
			     MLM_VGUEST_TEXT_VIDEO_NORMAL|
			     MLM_VGUEST_TEXT_COLOR_WHITE,
			     " ");
	  delay_warning_on=0;
	  break;
      }
      break;
  }
}
